@@ -12,12 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import dataclasses
 from functools import partial
 import typing as tp
 from typing import Any
+import warnings
+import dataclasses
 
 from flax import linen
+from flax import core
 from flax import nnx
 from flax.core import FrozenDict
 from flax.core import meta
@@ -26,9 +28,9 @@ from flax.nnx import variablelib
 from flax.nnx.bridge import variables as bv
 from flax.nnx.bridge import module as bdg_module
 from flax.nnx.module import Module
+from flax.nnx.statelib import State
 from flax.nnx.pytreelib import Pytree
 from flax.nnx.rnglib import Rngs
-from flax.nnx.statelib import State
 import jax
 from jax import tree_util as jtu
 
@@ -64,7 +66,7 @@ def functional(cls: tp.Type[M]) -> tp.Callable[..., Functional[M]]:
 
 
 def _set_initializing(module: Module, initializing: bool):
-  for k, value in graph.iter_graph(module):
+  for _, value in graph.iter_graph(module):
     if isinstance(value, Pytree):
       value._pytree__state._initializing = initializing
 
@@ -87,7 +89,13 @@ def lazy_init(fn: Module | tp.Callable[..., tp.Any], *args, **kwargs):
     _set_initializing(module, False)
   return fn
 
-class ToNNX(Module, pytree=False):
+def current_linen_module() -> linen.Module | None:
+  """Get the current Linen module from the Linen context."""
+  if linen.module._context.module_stack:  # pylint: disable=W0212
+    return linen.module._context.module_stack[-1]  # pylint: disable=W0212
+  return None
+
+class ToNNX(Module):
   """A wrapper to turn any Linen module into an NNX module.
 
   The result NNX module can be used standalone with all NNX APIs, or as a submodule of
@@ -118,12 +126,35 @@ class ToNNX(Module, pytree=False):
   """
 
   def __init__(
-    self,
-    module: linen.Module,
-    rngs: tp.Optional[Rngs] = None,
+      self,
+      module: linen.Module,
+      rngs: Rngs | jax.Array | None = None,
   ):
-    self.module = module
-    self.rngs = rngs
+    self.to_nnx__module = module
+
+    self.to_nnx__rngs: Rngs | None
+    if isinstance(rngs, jax.Array):
+      self.to_nnx__rngs = Rngs(params=rngs)
+    elif isinstance(rngs, nnx.Rngs):
+      self.to_nnx__rngs = rngs.fork()
+    else:
+      self.to_nnx__rngs = rngs
+
+  @property
+  def rngs(self) -> Rngs | None:
+    warnings.warn(
+        '`ToNNX.rngs` is deprecated. Please use `to_nnx__rngs` instead.',
+        DeprecationWarning,
+    )
+    return self.to_nnx__rngs
+
+  @property
+  def module(self) -> linen.Module:
+    warnings.warn(
+        '`ToNNX.module` is deprecated. Please use `to_nnx__module` instead.',
+        DeprecationWarning,
+    )
+    return self.to_nnx__module
 
   def lazy_init(self, *args, **kwargs):
     """A shortcut of calling `nnx.bridge.lazy_init()` upon this module."""
@@ -132,7 +163,7 @@ class ToNNX(Module, pytree=False):
   def __getattr__(self, name: str):
     if hasattr(super(), name):
       return super().__getattribute__(name)
-    maybe_method = getattr(self.module.__class__, name, None)
+    maybe_method = getattr(type(self.to_nnx__module), name, None)
     if callable(maybe_method):
       method = partial(self.__call__, method=maybe_method)
       method.__self__ = self
@@ -140,60 +171,71 @@ class ToNNX(Module, pytree=False):
     return super().__getattribute__(name)
 
   def __call__(
-    self, *args: Any, rngs: tp.Optional[Rngs] = None,
-    method: tp.Callable[..., Any] | str | None = None, **kwargs: Any
+    self,
+    *args: Any,
+    rngs: Rngs | jax.Array | None = None,
+    method: tp.Callable[..., Any] | str | None = None,
+    mutable: tp.Any = None,
+    **kwargs: Any,
   ) -> Any:
-
     # Shape-based lazy init of the flax variables
-    if not rngs:
-      rngs = self.rngs
+    if rngs is None:
+      rngs = self.to_nnx__rngs
+    if isinstance(rngs, nnx.Rngs):
+      _rngs = {name: stream() for name, stream in rngs.items()}
+    elif isinstance(rngs, jax.Array):
+      _rngs = {'params': rngs}
+    else:
+      _rngs = {}
+    # rename default to params
+    if 'params' not in _rngs and 'default' in _rngs:
+      _rngs['params'] = _rngs.pop('default')
     if self._pytree__state.initializing:
-      _rngs = (
-        {name: stream() for name, stream in rngs.items()} if rngs else {}
-      )
-      # rename default to params
-      if 'params' not in _rngs and 'default' in _rngs:
-        _rngs['params'] = _rngs.pop('default')
-      out, variables = self.module.init_with_output(_rngs, *args, method=method, **kwargs)
-
-      nnx_attrs = bv.linen_vars_to_nnx_attrs(variables)
-      for attr_name, value in nnx_attrs.items():
-        setattr(self, attr_name, value)
-
+      out, updates = self.to_nnx__module.init_with_output(_rngs, *args, method=method, **kwargs)
     else:
       nnx_attrs = {
-        k: v
-        for k, v in vars(self).items()
-        if k not in ['module', 'rngs'] and not k.startswith('_pytree__')
+          k: v
+          for k, v in vars(self).items()
+          if not k.startswith('to_nnx__') and not k.startswith('_pytree__')
       }
       variables = bv.nnx_attrs_to_linen_vars(nnx_attrs)
 
-      _rngs = (
-        {name: stream() for name, stream in rngs.items()} if rngs else {}
-      )
-
       # Get `mutable` from top level bridge.Module context if any
-      if (m := bdg_module.current_module()) is not None:
+      if mutable is not None:
+        pass
+      elif (m := bdg_module.current_module()) is not None:  # type: ignore[assignment]
         assert m.scope is not None
         mutable = m.scope.mutable
-        if 'mutable' in kwargs and kwargs['mutable'] != mutable:
-          raise ValueError(
-            f"Multiple `mutable` arguments detected: {mutable} at top level vs "
-            f"{kwargs['mutable']} in ToNNX() call")
-        kwargs['mutable'] = mutable
+      elif (m := current_linen_module()) is not None:  # type: ignore[assignment]
+        assert m.scope is not None
+        mutable = m.scope.mutable
+      else:
+        mutable = False
 
-      out = self.module.apply(variables, *args, rngs=_rngs, method=method, **kwargs)
+      out = self.to_nnx__module.apply(
+        variables, *args, rngs=_rngs, method=method, mutable=mutable, **kwargs
+      )
+
+      # Split out the updates if `mutable` is passed into the Flax module
+      if mutable is not False:
+        out, updates = out
+      else:
+        updates = None
 
     # Split out the updates if `mutable` is passed into the Flax module
-    if kwargs.get('mutable', False) != False:
-      out, updates = out
+    if updates:
       nnx_attrs = bv.linen_vars_to_nnx_attrs(updates)
+      # nnx.update(self, nnx_attrs)
+      # TODO(cgarciae): ideally we just do an update but currently dictionaries don't allow
+      # insertion of new keys, we need to enable this in NNX to simplify the code bellow
+      # to the simple nnx.update(self, nnx_attrs) above.
       for attr_name, value in nnx_attrs.items():
         if hasattr(self, attr_name) and isinstance(value, dict):
-          original_tree = getattr(self, attr_name)
-          setattr(self, attr_name, original_tree | value)
+          original_value = getattr(self, attr_name)
+          new_values = bv._recursive_merge(original_value, value)
+          setattr(self, attr_name, nnx.data(new_values))
         else:
-          setattr(self, attr_name, value)
+          setattr(self, attr_name, nnx.data(value))
 
     return out
 
@@ -210,6 +252,7 @@ def linen_rngs_dict(linen_module: linen.Module, add_default: bool = False):
   return rngs
 
 def _get_module_method(module, method: tp.Callable[..., Any] | str | None):
+  """Get a callable method from the module, or raise TypeError."""
   if method is None:
     method = '__call__'
 
@@ -264,27 +307,24 @@ class ToLinen(linen.Module):
       NNX module.
     skip_rng: True if this NNX module doesn't need `rngs` arg during
       initialization (not common).
-    abstract_init: if True (default) the NNX module will be initialized under
-      `nnx.eval_shape`, useful to minimize memory consumption, else it will be
-      initialized normally.
 
   Returns:
-    A Linen module that behaves the same as the wrapped NNX module.
+    A stateful NNX module that behaves the same as the wrapped Linen module.
   """
+
   nnx_class: tp.Callable[..., Module]
   args: tp.Sequence = ()
   kwargs: tp.Mapping[str, tp.Any] = FrozenDict({})
   skip_rng: bool = False
-  abstract_init: bool = True
-  metadata_fn: tp.Callable[[variablelib.Variable], tp.Any] | None = (
-      bv.to_linen_var
-  )
+  metadata_fn: tp.Callable[[variablelib.Variable], tp.Any] | None = bv.to_linen_var
 
   @linen.compact
-  def __call__(self, *args, nnx_method: tp.Callable[..., Any] | str | None = None, **kwargs):
-    module_kwargs = dict(self.kwargs)
-    maybe_add_default = not self.is_initializing()
+  def __call__(
+    self, *args, nnx_method: tp.Callable[..., Any] | str | None = None, **kwargs
+  ):
     def _module_kwargs():
+      maybe_add_default = not self.is_initializing()
+      module_kwargs = dict(self.kwargs)
       if not self.skip_rng:
         module_kwargs['rngs'] = nnx.Rngs(
             **linen_rngs_dict(self, add_default=maybe_add_default)
@@ -301,30 +341,35 @@ class ToLinen(linen.Module):
       out = method_fn(module, *args, **kwargs)
       return out
 
-    # create state
+    # create the nnx module
+    module = self.nnx_class(*self.args, **_module_kwargs())
+
+    # update nnx module from linen variables
     def maybe_unbox(x):
       if isinstance(x, meta.AxisMetadata):
         return x.unbox()
       return x
     states = jtu.tree_map(
         maybe_unbox,
-        list(self.variables.values()),
+        list(core.unfreeze(self.variables).values()),  # type: ignore[wrong-arg-types, arg-type]
         is_leaf=lambda x: isinstance(x, meta.AxisMetadata),
     )
     if not states:
       states = ({},)
 
-    # update module state
-    if self.abstract_init:
-      module = nnx.eval_shape(
-          lambda: self.nnx_class(*self.args, **_module_kwargs())
-      )
-    else:
-      module = self.nnx_class(*self.args, **_module_kwargs())
-    nnx.update(module, *states)
-    nnx.reseed(
-        module, **linen_rngs_dict(self, add_default=maybe_add_default)
-    )  # reseed with keys from linen apply call.
+    new_state = nnx.merge_state(*states)
+    new_state_flat = nnx.traversals.flatten_mapping(new_state)
+    current_state_flat = nnx.traversals.flatten_mapping(nnx.state(module))
+    unknown_state_flat = {path: v for path, v in new_state_flat.items() if path not in current_state_flat}
+
+    if unknown_state_flat:
+      paths_str = ""
+      for path, _ in unknown_state_flat.items():
+        paths_str += f"\n  - {'/'.join(map(str, path))}"
+
+      warnings.warn(f"Found unknown module paths in incoming state:{paths_str}")
+
+    nnx.update(module, new_state)
 
     method_fn = _get_module_method(module, nnx_method)
     out = method_fn(module, *args, **kwargs)
@@ -334,6 +379,8 @@ class ToLinen(linen.Module):
   def __getattr__(self, name: str):
     if hasattr(super(), name):
       return super().__getattribute__(name)
+    if name in self.kwargs:
+      return self.kwargs[name]
     maybe_method = getattr(self.nnx_class, name, None)
     if callable(maybe_method):
       method = partial(self.__call__, nnx_method=maybe_method)
@@ -345,7 +392,7 @@ class ToLinen(linen.Module):
     """Store the NNX module's graph def and state inside Linen module variables."""
     state = nnx.state(module, nnx.Not(nnx.RngState))
 
-    collection_flat_state: dict[str, list[tuple[tuple[str, ...], tp.Any]]] = {}
+    collection_flat_state: dict[str, list[tuple[tuple[tp.Any, ...], tp.Any]]] = {}
 
     # group state by collection
     for path, leaf in nnx.to_flat_state(state):
@@ -363,8 +410,8 @@ class ToLinen(linen.Module):
 
         def _to_linen_var(x):
           if isinstance(x, nnx.Variable):
-            if self.metadata_fn:
-              return self.metadata_fn(x)
+            if self.metadata_fn is not None:
+              return self.metadata_fn(x)  # pylint: disable=too-many-function-args
             else:
               return x.value
           return x
@@ -379,6 +426,14 @@ class ToLinen(linen.Module):
           self.put_variable(collection, k, v)
 
 
+
+class _Missing:
+  ...
+
+
+_MISSING = _Missing()
+
+
 def to_linen(
     nnx_class: tp.Callable[..., Module],
     *args,
@@ -397,6 +452,78 @@ def to_linen(
       kwargs=FrozenDict(kwargs),
       metadata_fn=metadata_fn,
       skip_rng=skip_rng,
-      abstract_init=abstract_init,
       name=name,
   )
+
+def to_linen_class(
+    base_nnx_class: type[M],
+    base_metadata_fn: tp.Callable[[variablelib.VariableState], tp.Any] | None = bv.to_linen_var,
+    base_skip_rng: bool = False,
+    **partial_kwargs: tp.Any,
+) -> type[ToLinen]:
+  """Dynamically wraps an NNX module class into a Flax Linen module class."""
+
+  class ToLinenPartial(ToLinen):
+    """A dynamically created Linen Module that wraps a specific NNX Module.
+
+    This class is not meant to be used directly. Instead, it is created and
+    returned by the `to_linen_class` function. It acts as a "partially applied"
+    version of the `ToLinen` wrapper, where the NNX module to be wrapped and
+    its default arguments are pre-configured.
+
+    When you instantiate this class, it behaves like a standard Linen module.
+    The arguments you provide during instantiation can override the defaults
+    that were set when this class was created by `to_linen_class`.
+
+    For example:
+      >>> from flax import linen as nn, nnx
+      >>> from maxtext.src.maxtext.layers import linears
+      >>> # Create a specialized Linen wrapper for linears.DenseGeneral
+      >>> LinenDenseGeneral = to_linen_class(linears.DenseGeneral)
+      >>> # Now, LinenDenseGeneral can be used like a regular Linen module
+      >>> class MyModel(nn.Module):
+      ...   def setup(self):
+      ...     # Instantiate the wrapped linears.DenseGeneral with its arguments
+      ...     self.dense = LinenDenseGeneral(
+      ...         in_features_shape=10, out_features_shape=5
+      ...     )
+      ...   def __call__(self, x):
+      ...     return self.dense(x)
+
+    Attributes:
+      (The attributes are dynamically set by the `ToLinen` parent class based
+       on the arguments provided during instantiation.)
+    """
+
+    def __init_subclass__(cls, **kwargs):
+      super().__init_subclass__(**kwargs)
+
+      def __init__(
+          self,
+          args=None,
+          kwargs=None,
+          nnx_class=None,
+          skip_rng=None,
+          metadata_fn=None,
+          name=_MISSING,
+          parent=_MISSING,
+          **other_kwargs,
+      ):
+        linen_kwargs = {}
+        if not isinstance(parent, _Missing):
+          linen_kwargs["parent"] = parent
+        if not isinstance(name, _Missing):
+          linen_kwargs["name"] = name
+        ToLinen.__init__(
+            self,
+            nnx_class=nnx_class or base_nnx_class,
+            args=args or (),
+            metadata_fn=metadata_fn or base_metadata_fn,
+            skip_rng=skip_rng or base_skip_rng,
+            kwargs=FrozenDict({**partial_kwargs, **(kwargs or {}), **other_kwargs}),
+            **linen_kwargs,
+        )
+
+      cls.__init__ = __init__
+
+  return ToLinenPartial
