@@ -21,6 +21,7 @@ import threading
 import typing as tp
 from abc import ABCMeta
 from copy import deepcopy
+import warnings
 
 from flax.nnx import variablelib
 import jax
@@ -42,10 +43,10 @@ from flax.typing import SizeBytes
 BUILDING_DOCS = 'FLAX_DOC_BUILD' in os.environ
 
 A = tp.TypeVar('A')
-O = tp.TypeVar('O', bound='Object')
+P = tp.TypeVar('P', bound='Pytree')
 
-DataTag = '__data__'
-Data = tp.Annotated[A, DataTag]
+DataAnnotation = '__data__'
+Data = tp.Annotated[A, DataAnnotation]
 Data.__doc__ = """Data marks attributes of a class as pytree data using type annotations.
 
 Data annotations must be used at the class level and will apply to all instances.
@@ -131,55 +132,50 @@ def register_data_type(type_: type, /) -> None:
 
     foo = Foo(42)
 
-    assert nnx.is_data_type(foo.a)  # True
+    assert nnx.is_data(foo.a)  # True
     assert jax.tree.leaves(foo) == [MyType(value=42)]
-
   """
   DATA_REGISTRY.add(type_)
 
 
-def is_data_type(value: tp.Any, /) -> bool:
+def is_data(value: tp.Any, /) -> bool:
   """Checks if a value is a registered data type.
 
   This function checks a the value is registered data type, which means it is
-  automatically recognized as pytree data when assigned to an Object attribute.
+  automatically recognized as data when assigned a :class:`flax.nnx.Pytree` attribute.
 
   Data types are:
-  - jax.Arrays
-  - np.ndarrays
-  - ArrayRefs
-  - Variables (Param, BatchStat, RngState, etc.)
-  - All graph nodes (Object, Module, Rngs, etc.)
-  - Any type registered with `nnx.register_data_type`
+
+  - ``jax.Array``
+  - ``np.ndarray``
+  - ``ArrayRef``
+  - Variables (:class:`flax.nnx.Param`, :class:`flax.nnx.BatchStat`, `nnx.RngState`, etc.)
+  - All graph nodes (:class:`flax.nnx.Object`, :class:`flax.nnx.Module`, :class:`flax.nnx.Rngs`, etc.)
+  - Any type registered with :func:`flax.nnx.register_data_type`
+  - Any pytree that contains at least one node or leaf element of the above
+
 
   Example::
 
-    from flax import nnx
-    import jax.numpy as jnp
-
-    module = nnx.Linear(1, 1, rngs=nnx.Rngs(0))
-    blocks = [module, module, module]
-
-    assert nnx.is_data_type(jnp.array(42))  # Arrays are data
-    assert nnx.is_data_type(nnx.Param(1))   # Variables are data
-    assert nnx.is_data_type(nnx.Rngs(0))    # Objects are data
-    assert nnx.is_data_type(module)         # Objects are data
-
-    assert not nnx.is_data_type(0.)         # float is not data
-    assert not nnx.is_data_type(1)          # int is not data
-    assert not nnx.is_data_type("hello")    # str is not data
-    assert not nnx.is_data_type(blocks)     # list is not data
+    >>> from flax import nnx
+    >>> import jax.numpy as jnp
+    ... # ------ DATA ------------
+    >>> assert nnx.is_data( jnp.array(0) )                      # Arrays
+    >>> assert nnx.is_data( nnx.Param(1) )                      # Variables
+    >>> assert nnx.is_data( nnx.Rngs(2) )                       # nnx.Pytrees
+    >>> assert nnx.is_data( nnx.Linear(1, 1,rngs=nnx.Rngs(0)) ) # Modules
+    ... # ------ STATIC ------------
+    >>> assert not nnx.is_data( 'hello' )                       # strings, arbitrary objects
+    >>> assert not nnx.is_data( 42 )                            # int, float, bool, complex, etc.
+    >>> assert not nnx.is_data( [1, 2.0, 3j, jnp.array(1)] )    # list, dict, tuple, pytrees
 
 
   Args:
     value: The value to check.
 
   Returns:
-    True if the value is a registered data type, False otherwise.
-
-
+    A string representing the attribute status.
   """
-
   return (
     graph.is_node_leaf(value)
     or graph.is_graph_node(value)
@@ -187,6 +183,45 @@ def is_data_type(value: tp.Any, /) -> bool:
   )
 
 
+StaticAnnotation = '__static__'
+Static = tp.Annotated[A, StaticAnnotation]
+Static.__doc__ = """Static marks attributes of a class as static using type annotations.
+Static annotations must be used at the class level and will apply to all instances.
+The usage of Static is recommended when type annotations are used already present
+or required e.g. for dataclasses.
+"""
+
+
+@dataclasses.dataclass(frozen=True, slots=True)
+class StaticAttr:
+  value: tp.Any
+
+
+def static(value: A, /) -> A:
+  """Annotates a an attribute as static.
+
+  The return value from `static` must be directly assigned to an Object
+  attribute
+  which will be registered as static attribute.
+
+  Example::
+
+    from flax import nnx
+
+    class Foo(nnx.Pytree):
+      def __init__(self, a, b):
+        self.a = nnx.static(a)  # pytree metadata
+        self.b = nnx.data(b)    # pytree data
+
+    foo = Foo("one", "two")
+
+    assert jax.tree.leaves(foo) == ["two"]
+
+  By default ``nnx.Pytree`` will ...
+  """
+  return StaticAttr(value)  # type: ignore[return-value]
+
+
 def _collect_stats(
   node: tp.Any, node_stats: dict[int, dict[type[Variable], SizeBytes]]
 ):
@@ -282,6 +317,15 @@ jax.tree_util.register_pytree_node(
 )
 
 
+def check_pytree(pytree):
+  """Checks if a pytree is valid."""
+  if not isinstance(pytree, Pytree):
+    raise TypeError(f'Expected a Pytree, got {type(pytree)}.')
+
+  for name, value in vars(pytree).items():
+    pytree._check_value(name, value, new_status=None)
+
+
 class PytreeMeta(ABCMeta):
   if not tp.TYPE_CHECKING:
 
@@ -293,20 +337,20 @@ class PytreeMeta(ABCMeta):
 
 ObjectMeta = PytreeMeta
 
-def _graph_node_meta_call(cls: tp.Type[O], *args, **kwargs) -> O:
+def _graph_node_meta_call(cls: tp.Type[P], *args, **kwargs) -> P:
   node = cls.__new__(cls, *args, **kwargs)
   vars_obj = vars(node)
   vars_obj['_pytree__state'] = PytreeState()
   vars_obj['_pytree__nodes'] = cls._pytree__nodes
   cls._pytree_meta_construct(node, *args, **kwargs)
-  # register possible new data attributes after initialization
-  for name, value in vars_obj.items():
-    if name not in vars_obj['_pytree__nodes']:
-      if any(
-        is_data_type(leaf)
-        for leaf in jax.tree.leaves(value, is_leaf=is_data_type)
-      ):
-        vars_obj['_pytree__nodes'] = vars_obj['_pytree__nodes'].union((name,))
+  if cls._pytree__is_pytree:
+    missing: dict[str, bool] = {}
+    for name, value in vars(node).items():
+      if name not in vars_obj['_pytree__nodes']:
+        missing[name] = is_data(value)
+    if missing:
+      vars_obj['_pytree__nodes'] = vars_obj['_pytree__nodes'].update(missing)
+    check_pytree(node)
 
   return node
 
@@ -341,17 +385,27 @@ class MutableArrayRepr(reprlib.Representable):
     yield reprlib.Attr('dtype', self.dtype)
 
 
+class AttributeStatus(tp.NamedTuple):
+  is_data: bool
+  explicit: bool
+
+
 class Pytree(reprlib.Representable, metaclass=PytreeMeta):
   """Base class for all NNX objects."""
 
   if tp.TYPE_CHECKING:
-    _pytree__nodes: frozenset[str]
+    _pytree__nodes: graph.HashableMapping[tp.Any, bool]
     _pytree__state: PytreeState
+    _pytree__is_pytree: bool
 
   def __init_subclass__(
-    cls, *, pytree: bool = config.flax_pytree_module, **kwargs
+      cls,
+      *,
+      pytree: bool = config.flax_pytree_module,
+      **kwargs,
   ) -> None:
     super().__init_subclass__(**kwargs)
+    cls._pytree__is_pytree = pytree
 
     graph.register_graph_node_type(
       type=cls,
@@ -363,17 +417,29 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
       init=cls._graph_node_init,  # type: ignore
     )
 
-    parent_nodes: tp.Iterable[str] = getattr(cls, '_pytree__nodes', ())
+    nodes: dict[str, bool] = dict(getattr(cls, '_pytree__nodes', ()))
+    nodes['_pytree__state'] = True
+    try:
+      type_hints = tp.get_type_hints(
+          cls, globals(), {cls.__name__: cls}, include_extras=True
+      )
+    except NameError:
+      type_hints = cls.__annotations__
+    # add annotation attributes
+    for name, type_ in type_hints.items():
+      if isinstance(type_, str):
+        if type_.startswith('nnx.Data'):
+          nodes[name] = True
+        elif type_.startswith('nnx.Static'):
+          nodes[name] = False
+      else:
+        type_metadata = getattr(type_, '__metadata__', ())
+        if DataAnnotation in type_metadata:
+          nodes[name] = True
+        elif StaticAnnotation in type_metadata:
+          nodes[name] = False
 
-    all_nodes: set[str] = set(parent_nodes)
-
-    all_nodes.add('_pytree__state')
-    # add DataTag attributes
-    type_: type
-    for name, type_ in cls.__annotations__.items():
-      if type_ != tp.ClassVar and DataTag in getattr(type_, '__metadata__', ()):
-        all_nodes.add(name)
-    cls._pytree__nodes = frozenset(all_nodes)
+    cls._pytree__nodes = graph.HashableMapping(nodes, copy=False)
 
     if pytree:
       jax.tree_util.register_pytree_with_keys(
@@ -389,10 +455,21 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
 
   # Backward compatibility with PR #4863
   @property
-  def _object__nodes(self) -> frozenset[str]:
+  def _object__nodes(self):
+    warnings.warn(
+      "'_object__nodes' is deprecated, use '_pytree__nodes' instead.",
+      DeprecationWarning,
+      stacklevel=2,
+    )
     return self._pytree__nodes
+
   @property
-  def _object__state(self) -> PytreeState:
+  def _object__state(self):
+    warnings.warn(
+      "'_object__state' is deprecated, use '_pytree__state' instead.",
+      DeprecationWarning,
+      stacklevel=2,
+    )
     return self._pytree__state
 
   if not tp.TYPE_CHECKING:
@@ -400,27 +477,144 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
     def __setattr__(self, name: str, value: Any) -> None:
       self._setattr(name, value)
 
-  def _setattr(self, name: str, value: tp.Any) -> None:
+  def _setattr(self, name, value: tp.Any) -> None:
     self._check_valid_context(
       lambda: f"Cannot mutate '{type(self).__name__}' from different trace level"
     )
+    data: bool = False
+    explicit: bool = False
     if type(value) is DataAttr:
       value = value.value
-      if name not in self._pytree__nodes:
-        self._pytree__nodes = self._pytree__nodes.union((name,))
-    # any attribute that contains known data types will be registered as data
-    elif name not in self._pytree__nodes and any(
-      is_data_type(leaf)
-      for leaf in jax.tree.leaves(value, is_leaf=is_data_type)
+      if self._pytree__is_pytree:
+        data = True
+        explicit = True
+    elif type(value) is StaticAttr:
+      value = value.value
+      if self._pytree__is_pytree:
+        data = False
+        explicit = True
+    elif self._pytree__is_pytree:
+      data = is_data(value)
+    if self._pytree__is_pytree:
+      self._check_value(name, value, AttributeStatus(data, explicit))
+      if name not in self._pytree__nodes or (
+          explicit and self._pytree__nodes[name] != data
+      ):
+        vars(self)['_pytree__nodes'] = self._pytree__nodes.update({name: data})
+    if isinstance(name, str):
+      object.__setattr__(self, name, value)
+    else:
+      vars(self)[name] = value
+
+  def _check_value(self, key, value, new_status: AttributeStatus | None):
+    def _has_arrays(leaves):
+      return any(
+          isinstance(leaf, (np.ndarray, jax.Array))
+          or variablelib.is_array_ref(leaf)
+          for leaf in leaves
+      )
+
+    def _get_annotations(leaves):
+      return {
+          'data' if type(leaf) is DataAttr else 'static'
+          for leaf in leaves
+          if type(leaf) is DataAttr or type(leaf) is StaticAttr
+      }
+
+    def _has_visited(x):
+      if id(x) in visited:
+        return True
+      visited.add(id(x))
+      return False
+
+    visited: set[int] = set()
+    leaves = jax.tree.leaves(value, is_leaf=_has_visited)
+    current_is_data = (
+        self._pytree__nodes[key] if key in self._pytree__nodes else False
+    )
+    existing_attr = key in vars(self)
+    if (
+        new_status is not None
+        and not new_status.explicit
+        and new_status.is_data
+        and existing_attr
+        and not current_is_data
     ):
-      self._pytree__nodes = self._pytree__nodes.union((name,))
-    object.__setattr__(self, name, value)
+      raise ValueError(
+          f"Cannot assign data value of type '{type(value)}' to static"
+          f" attribute '{key}' of Pytree type '{type(self)}'. To override the"
+          ' status explicitly wrap the value with nnx.data on assignment:\n\n '
+          f' _.{key} = nnx.data(...)\n\n'
+      )
+
+    if _has_arrays(leaves):
+      # check no data in nnx.static assignments
+      if new_status is not None:
+        if not new_status.is_data and new_status.explicit:
+          raise ValueError(
+              f"Found Arrays in value of type '{type(value)}' annotated with "
+              f"nnx.static(...) when setting attribute '{key}' of Pytree type "
+              f"'{type(self)}'."
+          )
+        if not new_status.explicit and not current_is_data and existing_attr:
+          base_pytree_type = Pytree
+          for t in type(self).mro()[1:]:
+            if issubclass(t, nnx.Pytree):
+              base_pytree_type = t
+              break
+          raise ValueError(
+              f"Found Arrays on value of type '{type(value)}' assigned to"
+              f" static attribute '{key}' of Pytree type '{type(self)}'. Static"
+              ' attributes should not contain Array values. Consider one of'
+              ' the following options:\n\n1. If the attribute is meant to be'
+              ' static, either remove the Array value or wrap it in a static'
+              ' container (e.g., using `nnx.static(...)`).\n2. If the'
+              ' attribute is meant to be data, wrap the value with nnx.data on'
+              f' assignment:\n\n  _.{key} = nnx.data(...)\n\n3. Alternatively,'
+              ' annotate the class attribute with nnx.Data:\n\n  class'
+              f' {type(self).__name__}({base_pytree_type.__name__}):\n   '
+              f' {key}: nnx.Data[{type(value).__name__}]\n\n4. Disable pytree'
+              ' for this class:\n\n  class'
+              f' {type(self).__name__}({base_pytree_type.__name__},'
+              ' pytree=False):\n\n'
+          )
+      # check no data in static attributes after __init__
+      elif not current_is_data:
+        base_pytree_type = Pytree
+        for t in type(self).mro()[1:]:
+          if issubclass(t, nnx.Pytree):
+            base_pytree_type = t
+            break
+        raise ValueError(
+            f'Found unexpected Arrays on value of type {type(value)} in static'
+            f" attribute '{key}' of Pytree type '{type(self)}'. This is an"
+            ' error starting from Flax version 0.12.0.\nConsider one of the'
+            ' following options:\n\n1. If the attribute is meant to be static,'
+            ' either remove the Array value or wrap it in a static'
+            ' container.\n2. Wrap the value with nnx.data on'
+            f' assignment:\n\n  _.{key} = nnx.data(...)\n\n3. Annotate the'
+            ' class attribute with nnx.Data:\n\n  class'
+            f' {type(self).__name__}({base_pytree_type.__name__}):\n    {key}:'
+            f' nnx.Data[{type(value).__name__}]\n\n4. If the container is a'
+            ' list or dict, try using nnx.List(...) or nnx.Dict(...)'
+            ' instead.\n5. Disable pytree for this class:\n\n  class'
+            f' {type(self).__name__}({base_pytree_type.__name__},'
+            f' pytree=False):\n\n'
+        )
+    if tags := _get_annotations(leaves):
+      raise ValueError(
+          f'Found unexpected tags {tags} on attribute'
+          f" '{type(self).__name__}.{key}'. Values from nnx.data(...)"
+          ' and\nnnx.static(...) should be assigned to nnx.Pytree attributes'
+          ' directly, they should not be inside other structures. Got value of'
+          f" type '{type(value)}' on Pytree of type '{type(self)}'."
+      )
 
   def _check_valid_context(self, error_msg: tp.Callable[[], str]) -> None:
     if not self._pytree__state.trace_state.is_valid():
       raise errors.TraceContextError(error_msg())
 
-  def __deepcopy__(self: O, memo=None) -> O:
+  def __deepcopy__(self: P, memo=None) -> P:
     graphdef, state = graph.split(self)
     graphdef = deepcopy(graphdef)
     state = deepcopy(state)
@@ -463,7 +657,7 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
       OBJECT_CONTEXT.seen_modules_repr.add(id(self))
 
       for name, value in vars(self).items():
-        if name.startswith('_'):
+        if str(name).startswith('_'):
           continue
 
         def to_shape_dtype(value):
@@ -518,7 +712,7 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
         first_line_annotation = None
       children = {}
       for name, value in vars(self).items():
-        if name.startswith('_'):
+        if str(name).startswith('_'):
           continue
         children[name] = value
 
@@ -550,16 +744,23 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
   # -------------------------
   # Pytree Definition
   # -------------------------
+  _pytree__key_sort_fn: tp.Callable | None = None
+
   def _pytree__flatten_with_paths(self):
     obj_vars = vars(self)
-    type_nodes = self._pytree__nodes
+    node_attributes = self._pytree__nodes
     node_names: list[str] = []
     node_attrs: list[tuple[tp.Any, tp.Any]] = []
     static_attrs: list[tuple[str, tp.Any]] = []
-    for name, value in sorted(obj_vars.items()):
-      if name in type_nodes:
+    for name, value in sorted(obj_vars.items(), key=self._pytree__key_sort_fn):
+      if name in node_attributes and node_attributes[name]:
         node_names.append(name)
-        node_attrs.append((jax.tree_util.GetAttrKey(name), value))
+        node_attrs.append((
+            jax.tree_util.GetAttrKey(name)
+            if isinstance(name, str)
+            else jax.tree_util.SequenceKey(name),
+            value,
+        ))
       else:
         static_attrs.append((name, value))
 
@@ -567,12 +768,12 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
 
   def _pytree__flatten(self):
     obj_vars = vars(self)
-    type_nodes = self._pytree__nodes
+    node_attributes = self._pytree__nodes
     node_names: list[str] = []
     node_attrs: list[tp.Any] = []
     static_attrs: list[tuple[str, tp.Any]] = []
-    for name, value in sorted(obj_vars.items()):
-      if name in type_nodes:
+    for name, value in sorted(obj_vars.items(), key=self._pytree__key_sort_fn):
+      if name in node_attributes and node_attributes[name]:
         node_names.append(name)
         node_attrs.append(value)
       else:
@@ -597,8 +798,8 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
   # Graph Definition
   # -------------------------
   def _graph_node_flatten(self):
-    nodes = vars(self).copy()
-    nodes = sorted(nodes.items())
+    nodes = vars(self)
+    nodes = sorted(nodes.items(), key=self._pytree__key_sort_fn)
     return nodes, type(self)
 
   def _graph_node_set_key(self, key: str, value: tp.Any):
@@ -619,7 +820,7 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
     return vars(self).pop(key)
 
   @staticmethod
-  def _graph_node_create_empty(node_type: tp.Type[O]) -> O:
+  def _graph_node_create_empty(node_type: tp.Type[P]) -> P:
     node = object.__new__(node_type)
     return node
 
@@ -629,4 +830,15 @@ class Pytree(reprlib.Representable, metaclass=PytreeMeta):
   def _graph_node_init(self, attributes: tp.Iterable[tuple[str, tp.Any]]):
     vars(self).update(attributes)
 
-Object = Pytree
\ No newline at end of file
+
+class Object(Pytree, pytree=False):
+  """Base class for NNX objects that are not pytrees."""
+
+  def __init_subclass__(cls, **kwargs):
+    pytree = kwargs.pop('pytree', False)
+    if pytree is not False:
+      raise ValueError(
+        "Object is not a pytree, but 'pytree' was explicitly set to "
+        f'{pytree!r} for type {cls}.'
+      )
+    super().__init_subclass__(pytree=pytree, **kwargs)
