@@ -72,11 +72,13 @@ def _collect_stats(
   ] = defaultdict(lambda: defaultdict())
   node_stats[id(node)] = ObjectInfo(path, stats, variable_groups)
 
-  if isinstance(node, nnx.Object):
+  if isinstance(node, nnx.Pytree):
     node._nnx_tabulate_id = id(node)  # type: ignore
     object_types.add(type(node))
 
-  node_dict = graph.get_node_impl(node).node_dict(node)
+  node_impl = graph.get_node_impl(node)
+  assert node_impl is not None
+  node_dict = node_impl.node_dict(node)
   for key, value in node_dict.items():
     if id(value) in node_stats:
       continue
@@ -203,7 +205,7 @@ def _call_obj(
     top_method(obj, *input_args, **input_kwargs)
   finally:
     for obj_type, methods in original_methods.items():
-      for name, top_method in methods.items():
+      for name, top_method in methods.items():  # type: ignore[assignment]
         setattr(obj_type, name, top_method)
 
 
@@ -332,9 +334,9 @@ def tabulate(
   _collect_stats((), obj, node_stats, object_types)
   _variable_types: set[type] = {
     nnx.RngState  # type: ignore[misc]
-    if issubclass(variable_state.type, nnx.RngState)
-    else variable_state.type
-    for _, variable_state in nnx.state(obj).flat_state()
+    if isinstance(leaf, nnx.RngState)
+    else type(leaf)
+    for _, leaf in nnx.to_flat_state(nnx.state(obj))
   }
   variable_types: list[type] = sorted(_variable_types, key=lambda t: t.__name__)
 
@@ -402,7 +404,7 @@ def tabulate(
             **metadata,
           }
         elif value_repr:
-          attributes[name] = value_repr
+          attributes[name] = value_repr  # type: ignore[assignment]
 
       if attributes:
         col_repr = _as_yaml_str(attributes) + '\n\n'
@@ -517,7 +519,12 @@ def _unflatten_to_simple_structure(xs: list[tuple[tuple[tp.Any, ...], tp.Any]]):
           else:
             cursor[key] = {}
       cursor = cursor[key]
-    cursor[path[-1]] = value
+    if isinstance(cursor, list):
+      assert path[-1] == len(cursor)
+      cursor.append(value)
+    else:
+      assert isinstance(cursor, dict)
+      cursor[path[-1]] = value
   return result
 
 def _as_yaml_str(value) -> str:
