@@ -57,7 +57,8 @@ import abc
 import copy
 import dataclasses
 import warnings
-from typing import Any, Callable
+from typing import Any
+from collections.abc import Callable
 
 import jax
 
