@@ -18,7 +18,8 @@ from __future__ import annotations
 import functools
 import inspect
 import warnings
-from typing import Any, Callable, Optional, Union, overload
+from typing import Any, overload
+from collections.abc import Callable
 
 import jax
 import jax.numpy as jnp
@@ -46,17 +47,18 @@ from flax.typing import (
 def dot_product_attention_weights(
     query: Array,
     key: Array,
-    bias: Optional[Array] = None,
-    mask: Optional[Array] = None,
+    bias: Array | None = None,
+    mask: Array | None = None,
     broadcast_dropout: bool = True,
-    dropout_rng: Optional[PRNGKey] = None,
+    dropout_rng: PRNGKey | None = None,
     dropout_rate: float = 0.0,
     deterministic: bool = False,
-    dtype: Optional[Dtype] = None,
+    dtype: Dtype | None = None,
     precision: PrecisionLike = None,
-    module: Optional[Module] = None,
+    module: Module | None = None,
     force_fp32_for_softmax: bool = False,
-    einsum_dot_general: Callable[..., Array] = jax.lax.dot_general,
+    einsum_dot_general: Callable[..., Array] | None = None,
+    einsum: Callable[..., Array] | None = None,
 ):
   """Computes dot-product attention weights given query and key.
 
@@ -91,10 +93,30 @@ def dot_product_attention_weights(
       fp32. This is useful for mixed-precision training where higher precision
       is desired for numerical stability.
     einsum_dot_general: the dot_general to use in einsum.
+    einsum: If unspecified, default `jnp.einsum` will be used. This argument is
+      mutually exclusive with `precision` and `einsum_dot_general`.
+
+  Raises:
+    ValueError: if both `precision`/`einsum_dot_general` and `einsum` are
+      specified.
 
   Returns:
     Output of shape ``[batch..., num_heads, q_length, kv_length]``.
   """
+  if (precision or einsum_dot_general) and einsum:
+    raise ValueError(
+        'precision/einsum_dot_general and einsum are mutually exclusive. Please'
+        ' specify only one of them.'
+    )
+  if not einsum:
+    einsum = functools.partial(
+        jnp.einsum,
+        precision=precision,
+        _dot_general=einsum_dot_general
+        if einsum_dot_general
+        else jax.lax.dot_general,
+    )
+
   query, key = promote_dtype(query, key, dtype=dtype)
   dtype = query.dtype
 
@@ -107,13 +129,7 @@ def dot_product_attention_weights(
   depth = query.shape[-1]
   query = query / jnp.sqrt(depth).astype(dtype)
   # attn weight shape is (batch..., num_heads, q_length, kv_length)
-  attn_weights = jnp.einsum(
-      '...qhd,...khd->...hqk',
-      query,
-      key,
-      precision=precision,
-      _dot_general=einsum_dot_general,
-  )
+  attn_weights = einsum('...qhd,...khd->...hqk', query, key)
 
   # apply attention bias: masking, dropout, proximity bias, etc.
   if bias is not None:
@@ -151,17 +167,19 @@ def dot_product_attention(
     query: Array,
     key: Array,
     value: Array,
-    bias: Optional[Array] = None,
-    mask: Optional[Array] = None,
+    bias: Array | None = None,
+    mask: Array | None = None,
     broadcast_dropout: bool = True,
-    dropout_rng: Optional[PRNGKey] = None,
+    dropout_rng: PRNGKey | None = None,
     dropout_rate: float = 0.0,
     deterministic: bool = False,
-    dtype: Optional[Dtype] = None,
+    dtype: Dtype | None = None,
     precision: PrecisionLike = None,
-    module: Optional[Module] = None,
+    module: Module | None = None,
     force_fp32_for_softmax: bool = False,
-    einsum_dot_general: Callable[..., Array] = jax.lax.dot_general,
+    einsum_dot_general: Callable[..., Array] | None = None,
+    qk_attn_weights_einsum: Callable[..., Array] | None = None,
+    attn_weights_value_einsum: Callable[..., Array] | None = None,
 ):
   """Computes dot-product attention given query, key, and value.
 
@@ -200,11 +218,39 @@ def dot_product_attention(
     force_fp32_for_softmax: bool, whether to force the softmax to be computed in
       fp32. This is useful for mixed-precision training where higher precision
       is desired for numerical stability.
-    einsum_dot_general: the dot_general to use in einsum.
+    einsum_dot_general: the dot_general to use in `jnp.einsum`.
+    qk_attn_weights_einsum: the einsum for computing the attention weights. When
+      unspecified, the default `jnp.einsum` will be used. This argument is
+      mutually exclusive with `precision` and `einsum_dot_general`.
+    attn_weights_value_einsum: the einsum for computing the product of the
+      attention weights and the values. When unspecified, the default
+      `jnp.einsum` will be used. This argument is mutually exclusive with
+      `precision` and `einsum_dot_general`.
 
   Returns:
     Output of shape ``[batch..., q_length, num_heads, v_depth_per_head]``.
+
+  Raises:
+    ValueError: if both `precision`/`einsum_dot_general` and
+    `qk_attn_weights_einsum`/`attn_weights_value_einsum` are
+      specified.
   """
+  if (qk_attn_weights_einsum and not attn_weights_value_einsum) or (
+      not qk_attn_weights_einsum and attn_weights_value_einsum
+  ):
+    raise ValueError(
+        'qk_attn_weights_einsum and attn_weights_value_einsum must be specified'
+        ' together.'
+    )
+  if (precision or einsum_dot_general) and (
+      qk_attn_weights_einsum or attn_weights_value_einsum
+  ):
+    raise ValueError(
+        'precision/einsum_dot_general and'
+        ' qk_attn_weights_einsum/attn_weights_value_einsum are mutually'
+        ' exclusive. Please specify only one of them.'
+    )
+
   query, key, value = promote_dtype(query, key, value, dtype=dtype)
   dtype = query.dtype
   assert key.ndim == query.ndim == value.ndim, 'q, k, v must have same rank.'
@@ -231,15 +277,21 @@ def dot_product_attention(
       module,
       force_fp32_for_softmax,
       einsum_dot_general=einsum_dot_general,
+      einsum=qk_attn_weights_einsum,
   )
-
+  if not attn_weights_value_einsum:
+    attn_weights_value_einsum = functools.partial(
+        jnp.einsum,
+        precision=precision,
+        _dot_general=einsum_dot_general
+        if einsum_dot_general
+        else jax.lax.dot_general,
+    )
   # return weighted sum over values for each query position
-  return jnp.einsum(
+  return attn_weights_value_einsum(
       '...hqk,...khd->...qhd',
       attn_weights,
       value,
-      precision=precision,
-      _dot_general=einsum_dot_general,
   )
 
 
@@ -319,16 +371,20 @@ class MultiHeadDotProductAttention(Module):
       num_heads, value_channels]``
     decode: Whether to prepare and use an autoregressive cache.
     normalize_qk: Should QK normalization be applied (arxiv.org/abs/2302.05442).
+    qk_attn_weights_einsum_cls: factory function to create the einsum for
+      computing the attention weights.
+    attn_weights_value_einsum_cls: factory function to create the einsum for
+      computing the product of the attention weights and the values.
   """
 
   num_heads: int
-  dtype: Optional[Dtype] = None
+  dtype: Dtype | None = None
   param_dtype: Dtype = jnp.float32
-  qkv_features: Optional[int] = None
-  out_features: Optional[int] = None
+  qkv_features: int | None = None
+  out_features: int | None = None
   broadcast_dropout: bool = True
   dropout_rate: float = 0.0
-  deterministic: Optional[bool] = None
+  deterministic: bool | None = None
   precision: PrecisionLike = None
   kernel_init: Initializer = default_kernel_init
   out_kernel_init: Initializer | None = None
@@ -340,21 +396,25 @@ class MultiHeadDotProductAttention(Module):
   normalize_qk: bool = False
   force_fp32_for_softmax: bool = False
   # Deprecated, will be removed.
-  qkv_dot_general: Optional[DotGeneralT] = None
-  out_dot_general: Optional[DotGeneralT] = None
+  qkv_dot_general: DotGeneralT | None = None
+  out_dot_general: DotGeneralT | None = None
   qkv_dot_general_cls: Any = None
   out_dot_general_cls: Any = None
+  qk_attn_weights_einsum_cls: Callable[..., Callable[..., Array]] | None = None
+  attn_weights_value_einsum_cls: Callable[..., Callable[..., Array]] | None = (
+      None
+  )
 
   @overload
   def __call__(
     self,
     inputs_q: Array,
-    inputs_k: Optional[Array] = None,
-    inputs_v: Optional[Array] = None,
+    inputs_k: Array | None = None,
+    inputs_v: Array | None = None,
     *,
-    mask: Optional[Array] = None,
-    deterministic: Optional[bool] = None,
-    dropout_rng: Optional[PRNGKey] = None,
+    mask: Array | None = None,
+    deterministic: bool | None = None,
+    dropout_rng: PRNGKey | None = None,
     sow_weights: bool = False,
   ):
     ...
@@ -364,10 +424,10 @@ class MultiHeadDotProductAttention(Module):
     self,
     inputs_q: Array,
     *,
-    inputs_kv: Optional[Array] = None,
-    mask: Optional[Array] = None,
-    deterministic: Optional[bool] = None,
-    dropout_rng: Optional[PRNGKey] = None,
+    inputs_kv: Array | None = None,
+    mask: Array | None = None,
+    deterministic: bool | None = None,
+    dropout_rng: PRNGKey | None = None,
     sow_weights: bool = False,
   ):
     ...
@@ -376,13 +436,13 @@ class MultiHeadDotProductAttention(Module):
   def __call__(
     self,
     inputs_q: Array,
-    inputs_k: Optional[Array] = None,
-    inputs_v: Optional[Array] = None,
+    inputs_k: Array | None = None,
+    inputs_v: Array | None = None,
     *,
-    inputs_kv: Optional[Array] = None,
-    mask: Optional[Array] = None,
-    deterministic: Optional[bool] = None,
-    dropout_rng: Optional[PRNGKey] = None,
+    inputs_kv: Array | None = None,
+    mask: Array | None = None,
+    deterministic: bool | None = None,
+    dropout_rng: PRNGKey | None = None,
     sow_weights: bool = False,
   ):
     """Applies multi-head dot product attention on the input data.
@@ -539,7 +599,7 @@ class MultiHeadDotProductAttention(Module):
         # update key, value caches with our new 1d spatial slices
         cur_index = cache_index.value
         zero = jnp.array(0, dtype=lax.dtype(cur_index.dtype))
-        indices: tuple[Union[int, jax.Array], ...] = (zero,) * len(
+        indices: tuple[int | jax.Array, ...] = (zero,) * len(
           batch_dims
         ) + (
           cur_index,
@@ -574,6 +634,19 @@ class MultiHeadDotProductAttention(Module):
     else:
       m_deterministic = True
 
+    # `qk_attn_weights_einsum` and `attn_weights_value_einsum` are optional
+    # arguments that can be used to override the default `jnp.einsum`. They
+    # exist for quantized einsum support in AQT.
+    qk_attn_weights_einsum = (
+        self.qk_attn_weights_einsum_cls()
+        if self.qk_attn_weights_einsum_cls
+        else None
+    )
+    attn_weights_value_einsum = (
+        self.attn_weights_value_einsum_cls()
+        if self.attn_weights_value_einsum_cls
+        else None
+    )
     # apply attention
     attn_args = (query, key, value)
     # This kwargs list match the default nn.dot_product_attention.
@@ -587,6 +660,8 @@ class MultiHeadDotProductAttention(Module):
       dtype=self.dtype,
       precision=self.precision,
       force_fp32_for_softmax=self.force_fp32_for_softmax,
+      qk_attn_weights_einsum=qk_attn_weights_einsum,
+      attn_weights_value_einsum=attn_weights_value_einsum,
     )
     attn_kwargs = {
         k: v
@@ -712,9 +787,9 @@ class SelfAttention(MultiHeadDotProductAttention):
   def __call__(  # type: ignore
     self,
     inputs_q: Array,
-    mask: Optional[Array] = None,
-    deterministic: Optional[bool] = None,
-    dropout_rng: Optional[PRNGKey] = None,
+    mask: Array | None = None,
+    deterministic: bool | None = None,
+    dropout_rng: PRNGKey | None = None,
     sow_weights: bool = False,
   ):
     """Applies multi-head dot product self-attention on the input data.
@@ -813,8 +888,8 @@ def make_causal_mask(
 
 
 def combine_masks(
-  *masks: Optional[Array], dtype: Dtype = jnp.float32
-) -> Optional[Array]:
+  *masks: Array | None, dtype: Dtype = jnp.float32
+) -> Array | None:
   """Combine attention masks.
 
   Args:
