@@ -35,9 +35,14 @@ from . import pytreelib as object
 from .pytreelib import Pytree as Pytree
 from .pytreelib import Object as Object
 from .pytreelib import Data as Data
+from .pytreelib import Static as Static
 from .pytreelib import data as data
+from .pytreelib import static as static
 from .pytreelib import register_data_type as register_data_type
-from .pytreelib import is_data_type as is_data_type
+from .pytreelib import is_data as is_data
+from .pytreelib import check_pytree as check_pytree
+from .helpers import Dict as Dict
+from .helpers import List as List
 from .helpers import Sequential as Sequential
 from .helpers import TrainState as TrainState
 from .module import M as M
@@ -64,6 +69,8 @@ from .graph import to_arrays as to_arrays
 from .graph import to_refs as to_refs
 from .graph import pure as pure
 from .graph import cached_partial as cached_partial
+from .graph import flatten as flatten
+from .graph import unflatten as unflatten
 from .nn import initializers as initializers
 from .nn.activations import celu as celu
 from .nn.activations import elu as elu
@@ -90,6 +97,7 @@ from .nn.activations import softplus as softplus
 from .nn.activations import standardize as standardize
 from .nn.activations import swish as swish
 from .nn.activations import tanh as tanh
+from .nn.activations import PReLU as PReLU
 from .nn.attention import MultiHeadAttention as MultiHeadAttention
 from .nn.attention import combine_masks as combine_masks
 from .nn.attention import dot_product_attention as dot_product_attention
@@ -115,6 +123,7 @@ from .nn.normalization import BatchNorm as BatchNorm
 from .nn.normalization import LayerNorm as LayerNorm
 from .nn.normalization import RMSNorm as RMSNorm
 from .nn.normalization import GroupNorm as GroupNorm
+from .nn.normalization import InstanceNorm as InstanceNorm
 from .nn.stochastic import Dropout as Dropout
 from .rnglib import Rngs as Rngs
 from .rnglib import RngStream as RngStream
