@@ -316,7 +316,16 @@ class HashableMapping(tp.Mapping[HA, HB], tp.Hashable):
     return len(self._mapping)
 
   def __hash__(self) -> int:
-    return hash(tuple(sorted(self._mapping.items())))
+    # use type-aware sorting to support int keys
+    def _pytree__key_sort_fn(item: tuple[tp.Any, tp.Any]) -> tuple[int, tp.Any]:
+      key, _ = item
+      if isinstance(key, int):
+        return (0, key)
+      elif isinstance(key, str):
+        return (1, key)
+      else:
+        raise ValueError(f'Unsupported key type: {type(key)!r}')
+    return hash(tuple(sorted(self._mapping.items(), key=_pytree__key_sort_fn)))
 
   def __eq__(self, other: tp.Any) -> bool:
     return (
@@ -326,6 +335,12 @@ class HashableMapping(tp.Mapping[HA, HB], tp.Hashable):
   def __repr__(self) -> str:
     return repr(self._mapping)
 
+  def update(self, other: tp.Mapping[HA, HB]) -> HashableMapping[HA, HB]:
+    """Updates the mapping with another mapping."""
+    mapping = dict(self._mapping)
+    mapping.update(other)
+    return HashableMapping(mapping, copy=False)
+
 
 @jax.tree_util.register_static
 @dataclasses.dataclass(frozen=True, repr=False)
@@ -1081,15 +1096,15 @@ def unflatten(  # type: ignore[invalid-annotation]
     state: A State instance.
     index_ref: A mapping from indexes to nodes references found during the graph
       traversal, defaults to None. If not provided, a new empty dictionary is
-      created. This argument can be used to unflatten a sequence of (graphdef, state)
-      pairs that share the same index space.
-    index_ref_cache: A mapping from indexes to existing nodes that can be reused.
-      When an reference is reused, ``GraphNodeImpl.clear`` is called to leave the
-      object in an empty state and then filled by the unflatten process, as a result
-      existing graph nodes are mutated to have the new content/topology
-      specified by the graphdef.
-    copy_variables: If True (default), variables in the state will be copied onto
-      the new new structure, else variables will be shared.
+      created. This argument can be used to unflatten a sequence of (graphdef,
+      state) pairs that share the same index space.
+    index_ref_cache: A mapping from indexes to existing nodes that can be
+      reused. When an reference is reused, ``GraphNodeImpl.clear`` is called to
+      leave the object in an empty state and then filled by the unflatten
+      process, as a result existing graph nodes are mutated to have the new
+      content/topology specified by the graphdef.
+    copy_variables: If True variables in the state will be copied onto the new
+      new structure, else variables will be shared. Default is False.
   """
   if isinstance(state, (State, dict)):
     leaves = _get_sorted_leaves(state)
@@ -2577,7 +2592,7 @@ def pop(
     return states
 
 
-def clone(node: Node) -> Node:
+def clone(node: Node, variables: bool = True) -> Node:
   """Create a deep copy of the given graph node.
 
   Example usage::
@@ -2591,11 +2606,13 @@ def clone(node: Node) -> Node:
 
   Args:
     node: A graph node object.
+    variables: If ``True`` (default) copies of the :class:`Variable` objects are created,
+      otherwise the Variables are shared between the original and cloned node.
   Returns:
     A deep copy of the :class:`Module` object.
   """
   graphdef, state = split(node)
-  return merge(graphdef, state, copy=True)
+  return merge(graphdef, state, copy=variables)
 
 
 def _mutable_like(path, x):
@@ -2612,6 +2629,7 @@ def to_arrays(
   """Converts a structure of array refs to regular arrays.
 
   Example::
+
     >>> from flax import nnx
     >>> import jax
     >>> import jax.numpy as jnp
@@ -2786,7 +2804,7 @@ def call(
   used to call methods on the underlying graph node. When a method is called, the
   output is returned along with a new (GraphDef, State) pair that represents the
   updated state of the graph node. ``call`` is equivalent to :func:`merge` > ``method``
-  > :func:`split`` but is more convenient to use in pure JAX functions.
+  > :func:`split` but is more convenient to use in pure JAX functions.
 
   Example::
 
@@ -2801,7 +2819,7 @@ def call(
     ...     self.count = Variable(jnp.array(0, dtype=jnp.uint32))
     ...
     ...   def increment(self):
-    ...     self.count += 1
+    ...     self.count.value += 1
     ...
     ...   def __call__(self, x):
     ...     self.increment()
@@ -2835,7 +2853,7 @@ def call(
     ...     self.count = nnx.Variable(jnp.array(0, dtype=jnp.uint32))
     ...
     ...   def increment(self):
-    ...     self.count += 1
+    ...     self.count.value += 1
     ...
     ...   def __call__(self, x):
     ...     self.increment()
@@ -2875,6 +2893,7 @@ def iter_graph(node: tp.Any, /) -> tp.Iterator[tuple[PathParts, tp.Any]]:
   root. Repeated nodes are visited only once. Leaves include static values.
 
   Example::
+
     >>> from flax import nnx
     >>> import jax.numpy as jnp
     ...
@@ -2890,7 +2909,7 @@ def iter_graph(node: tp.Any, /) -> tp.Iterator[tuple[PathParts, tp.Any]]:
     >>> for path, value in nnx.iter_graph(graph):
     ...   print(path, type(value).__name__)
     ...
-    (0, '_pytree__nodes') frozenset
+    (0, '_pytree__nodes') HashableMapping
     (0, '_pytree__state') PytreeState
     (0, 'b') Param
     (0, 'din') int
