@@ -27,6 +27,7 @@ import jax
 import treescope  # type: ignore[import-untyped]
 
 from flax import errors
+from flax.core import spmd as core_spmd
 from flax.nnx import filterlib, reprlib, tracers, visualization
 from flax.typing import Missing, PathParts, SizeBytes
 import jax.tree_util as jtu
@@ -76,6 +77,11 @@ def using_refs() -> bool:
     <...>
     >>> nnx.using_refs()
     True
+    >>> nnx.use_refs(False)
+    <...>
+    >>> nnx.using_refs()
+    False
+
 
   Returns:
     A boolean indicating if Variables are using ArrayRefs by default.
@@ -86,7 +92,6 @@ def using_refs() -> bool:
     return config.flax_array_ref
 
 
-
 def use_refs(value: bool, /):
   """Sets whether Variables should use ArrayRefs by default or not.
 
@@ -101,7 +106,9 @@ def use_refs(value: bool, /):
     >>> v.has_ref
     True
     >>> v.raw_value
-    ArrayRef(...)
+    Ref(...)
+    >>> nnx.use_refs(False)
+    <...>
 
   It can also be used as a context manager to temporarily
   change the default behavior for a block of code::
@@ -247,7 +254,7 @@ class Variable(tp.Generic[A], reprlib.Representable):
       if is_array_ref(value):
         _value = tp.cast(A, value)
       else:
-        _value = array_ref(jnp.asarray(value))  # type: ignore[assignment]
+        _value = array_ref(jnp.asarray(value))  # type: ignore[assignment]  # type: ignore[assignment]
     else:
       _value = value
 
@@ -273,7 +280,18 @@ class Variable(tp.Generic[A], reprlib.Representable):
 
     object.__setattr__(self, '_var_metadata', metadata)
     # run create_value hooks
-    object.__setattr__(self, 'raw_value', self.create_value(self.raw_value))
+    value = self.create_value(self.raw_value)
+
+    # shard the value if applicable
+    do_eager_sharding = config.flax_always_shard_variable
+    if 'eager_sharding' in metadata:
+      do_eager_sharding = metadata['eager_sharding']
+    if do_eager_sharding and 'sharding_names' in metadata:
+      value = core_spmd.shard_value(
+        value, metadata['sharding_names'], metadata.get('sharding_rules', None),
+        metadata.get('mesh', None))
+
+    object.__setattr__(self, 'raw_value', value)
 
   def __getattr__(self, name: str) -> tp.Any:
     if name in object.__getattribute__(self, '_var_metadata'):
@@ -329,8 +347,51 @@ class Variable(tp.Generic[A], reprlib.Representable):
   def has_ref(self) -> bool:
     return is_array_ref(self.raw_value)
 
-  def get_metadata(self):
-    return self._var_metadata
+  @tp.overload
+  def get_metadata(self) -> dict[str, tp.Any]: ...
+  @tp.overload
+  def get_metadata(self, name: str) -> tp.Any: ...
+  def get_metadata(self, name: str | None = None):
+    """Get metadata for the Variable.
+
+    Args:
+      name: The key of the metadata element to get. If not provided, returns
+        the full metadata dictionary.
+    """
+    if name is None:
+      return self._var_metadata
+    return self._var_metadata[name]
+
+  @tp.overload
+  def set_metadata(self, metadata: dict[str, tp.Any], /) -> None: ...
+  @tp.overload
+  def set_metadata(self, **metadata: tp.Any) -> None: ...
+  def set_metadata(self, *args, **kwargs) -> None:
+    """Set metadata for the Variable.
+
+    `set_metadata` can be called in two ways:
+
+    1. By passing a dictionary of metadata as the first argument, this will replace
+      the entire Variable's metadata.
+    2. By using keyword arguments, these will be merged into the existing Variable's
+      metadata.
+    """
+    if not self._trace_state.is_valid():
+      raise errors.TraceContextError(
+        f'Cannot mutate {type(self).__name__} from a different trace level'
+      )
+    if not (bool(args) ^ bool(kwargs)):
+      raise TypeError(
+        'set_metadata takes either a single dict argument or keyword arguments'
+      )
+    if len(args) == 1:
+      self._var_metadata = args[0]
+    elif kwargs:
+      self._var_metadata.update(kwargs)
+    else:
+      raise TypeError(
+        f'set_metadata takes either 1 argument or 1 or more keyword arguments, got args={args}, kwargs={kwargs}'
+      )
 
   def copy_from(self, other: Variable[A]) -> None:
     if type(self) is not type(other):
@@ -394,10 +455,12 @@ class Variable(tp.Generic[A], reprlib.Representable):
       self._var_metadata['on_remove_axis'](self, axis_index, axis_name)
 
   @tp.overload
-  def replace(self, value: B, **kwargs) -> Variable[B]: ...
+  def replace(self, value: B, **kwargs) -> Variable[B]:
+    ...
 
   @tp.overload
-  def replace(self, **kwargs) -> Variable[A]: ...
+  def replace(self, **kwargs) -> Variable[A]:
+    ...
 
   def replace(self, value: tp.Any = Missing, **kwargs) -> Variable[tp.Any]:
     if value is not Missing:
@@ -688,134 +751,82 @@ class Variable(tp.Generic[A], reprlib.Representable):
     return self.value.__ror__(other)  # type: ignore
 
   def __iadd__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__iadd__'):
-      value.__iadd__(other)
-    else:
-      self.value = value.__add__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value += x` instead.'
+    )
 
   def __isub__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__isub__'):
-      value.__isub__(other)
-    else:
-      self.value = value.__sub__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value -= x` instead.'
+    )
 
   def __imul__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__imul__'):
-      value.__imul__(other)
-    else:
-      self.value = value.__mul__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value *= x` instead.'
+    )
 
   def __imatmul__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__imatmul__'):
-      value.__imatmul__(other)
-    else:
-      self.value = value.__matmul__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value @= x` instead.'
+    )
 
   def __itruediv__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__itruediv__'):
-      value.__itruediv__(other)
-    else:
-      self.value = value.__truediv__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value /= x` instead.'
+    )
 
   def __ifloordiv__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__ifloordiv__'):
-      value.__ifloordiv__(other)
-    else:
-      self.value = value.__floordiv__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value //= x`` instead.'
+    )
 
   def __imod__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__imod__'):
-      value.__imod__(other)
-    else:
-      self.value = value.__mod__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value %= x` instead.'
+    )
 
   def __ipow__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__ipow__'):
-      value.__ipow__(other)
-    else:
-      self.value = value.__pow__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value **= x`` instead.'
+    )
 
   def __ilshift__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__ilshift__'):
-      value.__ilshift__(other)
-    else:
-      self.value = value.__lshift__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value <<= x`` instead.'
+    )
 
   def __irshift__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__irshift__'):
-      value.__irshift__(other)
-    else:
-      self.value = value.__rshift__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value >>= x`` instead.'
+    )
 
   def __iand__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__iand__'):
-      value.__iand__(other)
-    else:
-      self.value = value.__and__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value &= x` instead.'
+    )
 
   def __ixor__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__ixor__'):
-      value.__ixor__(other)
-    else:
-      self.value = value.__xor__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value ^= x` instead.'
+    )
 
   def __ior__(self: V, other) -> V:
-    if isinstance(other, Variable):
-      other = other.value
-    value = self.value
-    if hasattr(value, '__ior__'):
-      value.__ior__(other)
-    else:
-      self.value = value.__or__(other)
-    return self
+    raise NotImplementedError(
+      'In-place operations are no longer supported for Variable.\n'
+      'Use `variable.value |= x` instead.'
+    )
 
   def __neg__(self) -> A:
     return self.value.__neg__()  # type: ignore
@@ -867,13 +878,13 @@ class Variable(tp.Generic[A], reprlib.Representable):
 
 
 def _variable_flatten_with_keys(x: Variable[tp.Any]):
-  metadata = tuple(x.get_metadata().items())
+  metadata = tuple(sorted(x._var_metadata.items()))
   node = (jtu.GetAttrKey('value'), x.raw_value)
   return (node,), metadata
 
 
 def _variable_flatten(x: Variable[tp.Any]):
-  metadata = tuple(x.get_metadata().items())
+  metadata = tuple(sorted(x._var_metadata.items()))
   return (x.raw_value,), metadata
 
 
