@@ -68,6 +68,7 @@ PrecisionLike = Union[
 ]
 DotGeneralT = Callable[..., Array]
 ConvGeneralDilatedT = Callable[..., Array]
+EinsumT = Callable[..., Array]
 
 PaddingLike = Union[str, int, Sequence[Union[int, tuple[int, int]]]]
 LaxPadding = Union[str, Sequence[tuple[int, int]]]
@@ -168,11 +169,11 @@ MISSING = Missing()
 
 def _bytes_repr(num_bytes):
   count, units = (
-    (f'{num_bytes / 1e9 :,.1f}', 'GB')
+    (f'{num_bytes / 1e9:,.1f}', 'GB')
     if num_bytes > 1e9
-    else (f'{num_bytes / 1e6 :,.1f}', 'MB')
+    else (f'{num_bytes / 1e6:,.1f}', 'MB')
     if num_bytes > 1e6
-    else (f'{num_bytes / 1e3 :,.1f}', 'KB')
+    else (f'{num_bytes / 1e3:,.1f}', 'KB')
     if num_bytes > 1e3
     else (f'{num_bytes:,}', 'B')
   )
@@ -223,4 +224,13 @@ class SizeBytes:  # type: ignore[misc]
       if has_shape_dtype(leaf):
         size_bytes += cls.from_array(leaf)
 
-    return size_bytes
\ No newline at end of file
+    return size_bytes
+
+
+TupleArg = TypeVar('TupleArg', bound=tuple)
+
+
+class PromoteDtypeFn(Protocol):
+  def __call__(
+    self, args: TupleArg, /, *, dtype: Any = None, inexact: bool = True
+  ) -> TupleArg: ...
\ No newline at end of file
